/**********************************************************************/
/* MieruPC Project: dalaigon v1.0                Arch Lab. TOKYO TECH */
/**********************************************************************/
#include <mierulib.h>

#define WIDTH   10
#define LEFT    1
#define RIGHT   8

#define HEIGHT 16
#define TOP     2
#define BOTTOM 14

int block_prob[5];
char block_color[8] = {0x00, 0xe0, 0x1c, 0x4b, 0xfc, 0xe3, 0x92, 0xff};
char block_text[8][4] =
    {"   ", "<I>", "<O>", "<T>", "<V>", "<+>", "<X>", ">-<"}; 
char field[WIDTH][HEIGHT], last_field[WIDTH][HEIGHT];

int bx, by, br;
int bc[3], nextbc[3];
int tailx[4] = { 1,  1, -1, -1};
int taily[4] = {-1,  1,  1, -1};
int fall_time, lock_time, lock_max;
int score, level, erase, speed, chain, maxcolor, startlevel;
int topscore;
int highscore[7] = {5000, 4000, 3000, 2500, 2000, 1500, 1000};
int highlevel[7] = {10, 8, 6, 4, 3, 2, 1};
int higherase[7] = {80, 60, 50, 40, 30, 25, 20};
int chainmulti[20] = {1, 4, 8, 16, 24, 32, 48, 64, 80, 100, 120, 140,
                      160, 180, 200, 240, 280, 320, 360, 400};
int lengthbase[7] = {5, 10, 16, 23, 31, 40, 50};
int diagmulti;
int lasths, lastsc, lastlv, laster;

typedef struct {
    int left;
    int right;
    int rotl;
    int rotr;
    int drop;
    int rank;
    int help;
} inkey_t;

void showusage();

/**********************************************************************/
void my_usleep(int tm)
{
    volatile int *p = (volatile int *)0x010c;
    int end = (int) *p + tm / 10;
    while (*p < end);
}

/**********************************************************************/
void some_wait()
{
    my_usleep(15000);
}

/**********************************************************************/
void clearkey(inkey_t *key)
{
    key->left = key->right = key->rotl = 0;
    key->rotr = key->drop = key->rank = key->help = 0;
}

/**********************************************************************/
void updateacckey(inkey_t key, inkey_t *acckey)
{
    acckey->left  = (key.left)  ? acckey->left  + 1 : 0;
    acckey->right = (key.right) ? acckey->right + 1 : 0;
    acckey->rotl  = (key.rotl)  ? acckey->rotl  + 1 : 0;
    acckey->rotr  = (key.rotr)  ? acckey->rotr  + 1 : 0;
    acckey->drop  = (key.drop)  ? acckey->drop  + 1 : 0;
}

/**********************************************************************/
void getkey(inkey_t *key)
{
    key->rank  = (key->help  == 1) || (key->help  == 2);
    key->rank  = (key->rank  == 1) || (key->rank  == 2);
    key->drop  = (key->drop  == 1) || (key->drop  == 2);
    key->left  = (key->left  == 1) || (key->left  == 2);
    key->right = (key->right == 1) || (key->right == 2);
    key->rotl  = (key->rotl  == 1) || (key->rotl  == 2);
    key->rotr  = (key->rotr  == 1) || (key->rotr  == 2);

    unsigned int tmp_i;
    tmp_i = (unsigned int) *(volatile unsigned int *) 0x0100;
    while (tmp_i) {
        if      ((tmp_i >> 8) == 'H')
            key->help  = (key->help  == 2) ? 3 : 0;
        else if ((tmp_i >> 8) == 'R')
            key->rank  = (key->rank  == 2) ? 3 : 0;
        else if ((tmp_i >> 8) == 'S') 
            key->drop  = (key->drop  == 2) ? 3 : 0;
        else if ((tmp_i >> 8) == 'A') 
            key->left  = (key->left  == 2) ? 3 : 0;
        else if ((tmp_i >> 8) == 'D') 
            key->right = (key->right == 2) ? 3 : 0;
        else if ((tmp_i >> 8) == 'J') 
            key->rotl  = (key->rotl  == 2) ? 3 : 0;
        else if ((tmp_i >> 8) == 'K') 
            key->rotr  = (key->rotr  == 2) ? 3 : 0;
        else if ( tmp_i == 'H') key->help  = 2;
        else if ( tmp_i == 'R') key->rank  = 2;
        else if ( tmp_i == 'S') key->drop  = 2;
        else if ( tmp_i == 'A') key->left  = 2;
        else if ( tmp_i == 'D') key->right = 2;
        else if ( tmp_i == 'J') key->rotl  = 2;
        else if ( tmp_i == 'K') key->rotr  = 2;
        tmp_i = (unsigned int) *(volatile unsigned int *) 0x0100;
    }
}

/**********************************************************************/
void drawfield()
{
    int i, j, locate, col, lcol;
    lcol = -1;
    for (j = TOP; j <= BOTTOM; j++) {
        locate = 1;
        for (i = LEFT; i <= RIGHT; i++) {
            if (field[i][j] != last_field[i][j]) {
                last_field[i][j] = field[i][j];
                if (locate)
                    lcd_locate((i - LEFT) * 3 + 1, j - TOP + 1);
                col = field[i][j];
                if (col & 0x8)
                    col = 6;
                if ((lcol != col) && (col))
                    lcd_setcolor(block_color[col]);
                lcol = col;
                lcd_printf(block_text[field[i][j] & 0x7]);
                locate = 0;
            } else {
                locate = 1;
            }
        }
    }
    lcd_setcolor(COL_WHITE);
}

/**********************************************************************/
void drawwall() {
    int i;
	lcd_setcolor(COL_WHITE);
    for (i = 0; i <= BOTTOM - TOP + 2; i++) {
        lcd_locate(0, i);
        lcd_printf("#");
        lcd_locate((RIGHT - LEFT + 1) * 3 + 1, i);
        lcd_printf("#");
    }
    lcd_locate(0, 0);
    for (i = 0; i <= (RIGHT - LEFT + 1) * 3 + 1; i++)
        lcd_printf("#");
    lcd_locate(0, BOTTOM - TOP + 2);
    for (i = 0; i <= (RIGHT - LEFT + 1) * 3 + 1; i++)
        lcd_printf("#");
}

/**********************************************************************/
void drawnext()
{
    int i;
    for (i = 0; i < 3; i++) {
        lcd_locate(28 + i / 2 * 3, i != 1);
        lcd_setcolor(block_color[nextbc[i]]);
        lcd_printf(block_text[nextbc[i]]);
    }
    lcd_setcolor(COL_WHITE);
}

/**********************************************************************/
void drawstate()
{
    if (topscore != lasths) {
        lcd_locate(31, 7);
        lcd_printf("%8d", topscore);
    }
    if (score != lastsc) {
        lcd_locate(31, 8);
        lcd_printf("%8d", score);
    }
    if (level != lastlv) {
        lcd_locate(31, 11);
        lcd_printf("%8d", level);
    }
    if (erase != laster) {
        lcd_locate(31, 12);
        lcd_printf("%8d", erase);
    }
    lasths = topscore;
    lastsc = score;
    lastlv = level;
    laster = erase;
}

/**********************************************************************/
void drawtitleblock(char *str, int colmode)
{
    char printtable[6][6] = {"/", "=", "/=/", "/===/", "/ /", "==="};
    int lentable[6] = {1, 1, 3, 5, 3, 3};
    int x, y, ly, mode;
    lcd_cls();
    lcd_setcolor(COL_WHITE);
    x = y = ly = mode = 0;
    while (*str) {
        if (*str >= 'A') {
            if (mode) {
                lcd_printf("/");
                x++;
            }
            x += *str - 'A' + 1;
            y += x / 40;
            x %= 40;
            
            lcd_locate(x, y);
            mode = 1;
            if (y / 2 != ly / 2)
                lcd_setcolor(0xff - (y / 2) * (1 + colmode * 8));
            ly = y;
        } else if ((*str >= '1') && (*str <= '6')) {
            mode = 0;
            lcd_printf(printtable[*str - '1']);
            x += lentable[*str - '1'];
        }
        str++;
    }
    lcd_setcolor(COL_WHITE);
}

/**********************************************************************/
int title()
{
    int i, flg;
    inkey_t key, lastkey;
    score = 0;
    drawtitleblock(
        "HGGVG\\3B3BB3BB3B3B3E5B5BB5BB5B5B5E3B16AB16AB3B3B5\\e22P", 0);
    lcd_locate(23, 7);
    lcd_printf("dalaigon v1.0");
    lcd_locate(25, 8);
    lcd_printf("for MieruPC2008");
    lcd_locate(10, 10);
    lcd_printf("HIGH SCORE:%9d", highscore[0]);
    lcd_locate(10, 11);
    lcd_printf("START LV.:");
    lcd_locate(10, 13);
    lcd_printf("A/D: Start Level -/+");
    lcd_locate(6, 14);
    lcd_printf("H: Help  R: Ranking  J: Start");
    lcd_copypage(1, 0);
    clearkey(&key);
    getkey(&lastkey);
    flg = 0;
    while (1) {
        some_wait();
        getkey(&key);
        if ((key.left)  && (!lastkey.left))
            startlevel = (startlevel == 0) ? 0 : startlevel - 10;
        if ((key.right) && (!lastkey.right))
            startlevel = (startlevel == 50) ? 50 : startlevel + 10;
        if ((key.rotl)  && (!lastkey.rotl))
            return 0;
        if ((key.rank)  && (!lastkey.rank))
            return 1;
        if ((key.help)  && (!lastkey.help))
            return 2;
        lcd_locate(21, 11);
        lcd_printf("      ");
        lcd_locate(21, 11);
        for (i = 0; i <= startlevel; i += 10)
            lcd_printf((flg) ? "*" : "+");
        lcd_locate(28, 11);
        lcd_printf("%2d", startlevel);
        lcd_copypage(1, 0);
        lastkey = key;
        flg = 1 - flg;
    }
}

/**********************************************************************/
int randomblock()
{
    int i, num, ret, total;
    volatile int *p = (volatile int *)0x010c;

    total = 0;
    for (i = 0; i < maxcolor; i++)
        total += block_prob[i];
    num = (int)*p % total;
    for (i = 0; i < maxcolor; i++)
        if ((num -= block_prob[i]) < 0)
            break;
    ret = i + 1;
    block_prob[i]--;
    if (--total <= 4 * maxcolor)
        for (i = 0; i < maxcolor; i++)
            block_prob[i]++;
    return ret;
}

/**********************************************************************/
void fallingblock(int disp)
{
    field[bx][by] = (disp) ? bc[0] : 0;
    field[bx][by + taily[br]] = (disp) ? bc[1 + br % 2] : 0;
    field[bx + tailx[br]][by] = (disp) ? bc[2 - br % 2] : 0;
}

/**********************************************************************/
int checkhit(int x, int y, int r)
{
    if (y == 0)
        return 1; 
    if (field[x][y])
        return 1;
    if (field[x + tailx[r]][y])
        return 1;
    if (field[x][y + taily[r]])
        return 1;
    return 0;
}

/**********************************************************************/
void getspeed()
{
    int base[10] = {1, 2, 4, 6, 20, 3, 6, 10, 20, 30};
    int add[10] = {0, 0, 10, 20, 30, 40, 60, 80, 100, 120};
    maxcolor = 4 + (level >= 10);
    lock_max = (level < 50) ? 35 : 60 - level / 2;
    speed = base[level % 10] + add[level / 10];
    if (speed > 120)
        speed = 120;
}

/**********************************************************************/
void moveblock(int dir)
{
    fallingblock(0);
    if (!checkhit(bx + dir, by, br))
        bx += dir;
    fallingblock(1);
}

/**********************************************************************/
void rotateblock(int dir)
{
    int destbr = (br + dir + 4) % 4;
    fallingblock(0);
    if (!checkhit(bx, by, destbr)) {
        br = destbr;
    } else if (!checkhit(bx - tailx[destbr], by, destbr)) {
        bx -= tailx[destbr];
        br = destbr;
    } else if (!checkhit(bx, by - taily[destbr], destbr)) {
        by -= taily[destbr];
        br = destbr;
    }
    fallingblock(1);
}

/**********************************************************************/
void fallblock()
{
    fall_time += speed;
    while (fall_time >= 60) {
        fallingblock(0);
        if (!checkhit(bx, by + 1, br)) {
            by++;
            fall_time = (fall_time > 60) ? fall_time - 60 : 0;
        } else {
            fall_time = 59;
            lock_time++;
        }
        fallingblock(1);
    }
}

/**********************************************************************/
void dropblock()
{
    fall_time = 0;
    fallingblock(0);
    if (!checkhit(bx, by + 1, br)) {
        by++;
        score += diagmulti;
        score = (score > 99999999) ? 99999999 : score;
        topscore = (score > topscore) ? score : topscore;
    } else {
        lock_time = 99;
    }
    fallingblock(1);
}

/**********************************************************************/
int generateblock(int rot)
{
    int i, ret;
    for (i = 0; i < 3; i++) {
        bc[i] = nextbc[i];
        nextbc[i] = randomblock();
    }
    by = TOP;
    bx = LEFT + 3;
    br = 0;
    if (chain) {
        chain = 0;
        lcd_locate(27, 9);
        lcd_printf("dalaigon x%2d", diagmulti);
    }
    ret = (field[bx][by] == 0) && (field[bx + 1][by] == 0);
    if ((field[bx - 1][by]) && (rot == 3))
        bx--;
    fall_time = 0;
    lock_time = 0;
    if (rot)
        rotateblock(rot);
    fallingblock(1);
    drawnext();
    drawfield();
    return ret;
}

/**********************************************************************/
int blocklocked()
{
    int i;
    if (lock_time < lock_max)
        return 0;

    field[bx][by] = bc[0] | 0x8;
    field[bx][by + taily[br]] = bc[1 + br % 2] | 0x8;
    field[bx + tailx[br]][by] = bc[2 - br % 2] | 0x8;
    lcd_locate(0, 14);
    drawfield();
    lcd_copypage(1, 0);
    for (i = 0; i < 3; i++)
        some_wait();

    field[bx][by] = bc[0] ;
    field[bx][by + taily[br]] = bc[1 + br % 2];
    field[bx + tailx[br]][by] = bc[2 - br % 2];
    drawfield();
    lcd_copypage(1, 0);
    for (i = 0; i < 3; i++)
        some_wait();
    return 1;
}

/**********************************************************************/
int dropfield()
{
    int i, j;
    int ret = 0;
    for (i = LEFT; i <= RIGHT; i++) {
        for (j = BOTTOM; j >= 0; j--)
            if (field[i][j] == 0)
                break;
        for (; j >= 0; j--)
            if (field[i][j] != 0)
                break;
        if (j < 0)
            continue;
        ret = 1;
        for (; j >= 0; j--)
            field[i][j + 1] = field[i][j];
        field[i][0] = 0;
    }
    drawfield();
    lcd_copypage(1, 0);
    for (i = 0; i < 3; i++)
        some_wait();
    return ret;
}

/**********************************************************************/
int checkline(int x, int y, int dx, int dy, int *base)
{
    int checklog[(WIDTH > HEIGHT) ? WIDTH : HEIGHT];
    int i, ret, mode, count, lastcolor;
    ret = 0;
    count = 0;
    lastcolor = -1;
    for (i = 0; (x >= LEFT) && (x <= RIGHT) && (y <= BOTTOM);
         i++, x+=dx, y+=dy) {
        count = ((field[x][y] && (lastcolor == (field[x][y] & 0x07))) ?
                 count + 1 : 1);
        checklog[i] = count;
        lastcolor = field[x][y] & 0x07;
    }
    mode = 0;
    for (i--, x-=dx, y-=dy; i >= 0; i--, x-=dx, y-=dy) {
        if ((!mode) && (checklog[i] >= 3)) {
            mode = 1;
            *base += lengthbase[
                (checklog[i] > 9) ? 6 : checklog[i] - 3];
            ret++;
        }
        if (mode)
            field[x][y] |= 0x10;
        if (checklog[i] == 1)
            mode = 0;
    }
    
    return ret;  
}

/**********************************************************************/
int checkerase()
{
    int i, j, base, er, hvline, dline;
    char tmpstr1[13], tmpstr2[13];
    tmpstr1[12] = tmpstr2[12] = '\0';

    base = er = hvline = dline = 0;
    for (i = LEFT; i <= RIGHT; i++)
        hvline += checkline(i, TOP, 0, 1, &base);
    for (i = TOP; i <= BOTTOM; i++)
        hvline += checkline(LEFT, i, 1, 0, &base);
    for (i = LEFT; i <= RIGHT; i++) {
        dline += checkline(i, TOP, 1, 1, &base);
        dline += checkline(i, TOP, -1, 1, &base);
    }
    for (i = TOP + 1; i <= BOTTOM; i++) {
        dline += checkline(LEFT, i, 1, 1, &base);
        dline += checkline(RIGHT, i, -1, 1, &base);
    }

    if (!(hvline + dline))
        return 0;

    diagmulti = diagmulti + dline - hvline;
    diagmulti = (diagmulti < 1) ? 1 : (diagmulti > 20) ? 20 : diagmulti;

    for (i = LEFT; i <= RIGHT; i++)
        for (j = TOP; j <= BOTTOM; j++)
            if (field[i][j] & 0x10) {
                field[i][j] = 0x7;
                er++;
            }
    base *= chainmulti[chain];
    base *= level + 10;
    base *= hvline + dline;

    erase += er;
    erase = (erase > 99999) ? 99999 : erase;
    level = erase / 16 + startlevel;
    level = (level > 99) ? 99 : level;
    score += base * diagmulti;
    score = (score > 99999999) ? 99999999 : score;
    topscore = (score > topscore) ? score : topscore;
    chain++;
    chain = (chain > 20) ? 20 : chain;
    getspeed();

    snprintf(tmpstr1, 13, "%d x %d", base, diagmulti);
    snprintf(tmpstr2, 13, "%12s", tmpstr1);
    lcd_locate(27, 9);
    lcd_printf(tmpstr2);
    drawfield();
    lcd_copypage(1, 0);
    my_usleep(100000);

    for (i = LEFT; i <= RIGHT; i++)
        for (j = TOP; j <= BOTTOM; j++)
            if (field[i][j] == 0x7)
                field[i][j] = 0;
    drawfield();
    lcd_copypage(1, 0);
    my_usleep(300000);

    drawstate();
    snprintf(tmpstr1, 13, "%d", base * diagmulti);
    snprintf(tmpstr2, 13, "%12s ", tmpstr1);
    lcd_locate(27, 9);
    lcd_printf(tmpstr2);

    lcd_copypage(1, 0);
    my_usleep(200000);
    return 1;
}

/**********************************************************************/
void game_init()
{
    int i, j;
    score = erase = 0;
    diagmulti = 1;
    level = startlevel;
    getspeed();
    topscore = highscore[0];
    lasths = lastsc = laster = lastlv = -1;

    // clean up field
    lcd_cls();
    for (i = 0; i < HEIGHT; i++)
        for (j = 0; j < WIDTH; j++)
            field[j][i] = ((i > BOTTOM) || (j < LEFT) || 
                           (j > RIGHT)) ? 9 : 0;

    // show usage
    lcd_locate(33, 0);
    lcd_printf("NEXT");
    lcd_locate(28, 3);
    lcd_printf("A/D: move");
    lcd_locate(28, 4);
    lcd_printf("J/K: rotate");
    lcd_locate(28, 5);
    lcd_printf("  S: drop");
    lcd_locate(27, 7);
    lcd_printf("HI:");
    lcd_locate(27, 8);
    lcd_printf("SC:");
    lcd_locate(27, 9);
    lcd_printf("dalaigon x 1");
    lcd_locate(27, 11);
    lcd_printf("LV:");
    lcd_locate(27, 12);
    lcd_printf("ER:");
    lcd_locate(28, 14);
    lcd_printf("MieruPC-2008");
    drawstate();
    drawwall();

    for (i = 0; i < 5; i++)
        block_prob[i] = 5;
    for (i = 0; i < 3; i++)
        nextbc[i] = randomblock();
    drawnext();
    drawfield();
    lcd_copypage(1, 0);
    for (i = 0; i < 6; i++) {
        lcd_locate(11, 5);
	lcd_printf((i % 2 == 1) ? "     " :
		   (i == 0) ? "READY" :
		   (i == 2) ? " SET " : " GO! ");
	lcd_copypage(1, 0);
	my_usleep(400000);
    }
}

/**********************************************************************/
void gameover()
{
    int i, j, ret;
    inkey_t key;

    for (j = 0; j <= BOTTOM; j++) {
        for (i = LEFT; i <= RIGHT; i++)
            field[i][j] = (field[i][j]) ? 6 : 0;
        drawfield();
        lcd_copypage(1, 0);
        some_wait();
    }
    ret = 0;
    while (!ret) {
        for (i = LEFT; i <= RIGHT; i++)
            field[i][BOTTOM] = 0;
        ret = !dropfield();
    }
    for (i = BOTTOM - TOP; i >= 6; i--) {
        lcd_locate(8, i);
        lcd_printf("GAME OVER!");
        lcd_copypage(1, 0);
        some_wait();
        lcd_locate(8, i);
        lcd_printf("          ");
    }
    while (1) {
        some_wait();
        getkey(&key);
        if (key.rotl)
            break;
    }
}

/**********************************************************************/
void game()
{
    inkey_t key, acckey;
    int i;
    game_init();
    getkey(&acckey);
    generateblock((acckey.rotl) ? 3 : ((acckey.rotr) ? 1 : 0));
    while (1) {
        some_wait();
        getkey(&key);
        if ((key.rotl) && (acckey.rotl == 0))
            rotateblock(-1);
        if ((key.rotr) && (acckey.rotr == 0))
            rotateblock(1);
        if ((key.left) && ((acckey.left == 0) || (acckey.left > 8)))
            moveblock(-1);
        if ((key.right) && ((acckey.right == 0) || (acckey.right > 8)))
            moveblock(1);
        fallblock();
        if (key.drop)
            dropblock();
        
        if (blocklocked()) {
            do {
                while (dropfield());
            } while (checkerase());
            for (i = 0; i < 15; i++) {
                updateacckey(key, &acckey);
                some_wait();
                getkey(&key);
            }
            if (!generateblock((key.rotl) ? 3 : ((key.rotr) ? 1 : 0)))
                break;
        }
        drawfield();
        drawstate();
        lcd_copypage(1, 0);
        updateacckey(key, &acckey);
    }
    gameover();
}

/**********************************************************************/
int myrank()
{
    int rank, i;
    for (rank = 0; rank < 7; rank++)
        if (score > highscore[rank])
            break;
    if (rank == 7)
        return -1;
    for (i = 5; i >= rank; i--) {
        highscore[i + 1] = highscore[i];
        higherase[i + 1] = higherase[i];
        highlevel[i + 1] = highlevel[i];
    }
    highscore[rank] = score;
    higherase[rank] = erase;
    highlevel[rank] = level;
    return rank;
}

/**********************************************************************/
void ranking()
{
    int rank = myrank();
    int i;
    inkey_t key, lastkey;
    drawtitleblock("G262B4B4D4MDCBCDCMDCB4HMDCBLMD4BLI", 1);

    lcd_locate(9, 6);
    lcd_printf("#     SCORE  LV  BLOCK");
    for (i = 0; i < 7; i++) {
        lcd_locate(9, 7 + i);
        lcd_printf("%d  %8d  %2d  %5d", i + 1,
                   highscore[i], highlevel[i], higherase[i]);
    }
    lcd_locate(12, 14);
    lcd_printf("[[Press j key]]");
    clearkey(&key);
    getkey(&lastkey);
    while (1) {
        some_wait();
        getkey(&key);
        if (rank != -1) {
            i++;
            if (i > 5)
                i = 0;
            lcd_setcolor(block_color[i + 1]);
            lcd_locate(9, 7 + rank);
            lcd_printf("%d  %8d  %2d  %5d", rank + 1,
                       highscore[rank], highlevel[rank], higherase[rank]);
        }
        if ((key.rotl) && (!lastkey.rotl))
            return;
        lcd_copypage(1, 0);
        lastkey = key;
    }
}

/**********************************************************************/
void help()
{
    inkey_t key, lastkey;
    lcd_cls();
    lcd_locate(0, 0);
    lcd_setcolor(COL_YELLOW);
    lcd_printf("dalaigon ");
    lcd_setcolor(COL_WHITE);
    lcd_printf("is a falling-block puzzle game.");
    lcd_printf("Press A/D to move blocks left/right, K/J");
    lcd_printf("to rotate them, and S to drop them.     ");
    lcd_nextline();

    lcd_printf("When 3 or more blocks of the same kind  ");
    lcd_printf("are connected horizontally, vertically, ");
    lcd_printf("or diagonally, they will disappear.     ");
    lcd_nextline();

    lcd_printf("To earn more points, keep the ");
    lcd_setcolor(COL_CYAN);
    lcd_printf("dalaigon  ");
    lcd_printf("score multiplier ");
    lcd_setcolor(COL_WHITE);
    lcd_printf("higher. It's increased ");
    lcd_printf("when the block disappear diagonally (up ");
    lcd_printf("to x20), but decreased by horizontally  ");
    lcd_printf("or verticallyu erasion.");
    lcd_locate(12, 14);
    lcd_printf("[[Press j key]]");
    lcd_copypage(1, 0);
    clearkey(&key);
    getkey(&lastkey);
    while ((!key.rotl) || (lastkey.rotl)) {
        some_wait();
        lastkey = key;
        getkey(&key);
    }
}

/**********************************************************************/
void init()
{
    lcd_ttyopen(1);
    lcd_setdrawpage(1);
    lcd_cls();
    startlevel = 0;
}

/**********************************************************************/
void fini()
{
    lcd_ttyclose();
}

/**********************************************************************/
int main(int argc, char **argv)
{
    int phase = 0;

    init();
    while (1) {
        if (phase == 0) {
            phase += title() + 1;
        } else if (phase == 1) {
            game();
            phase = 2;
        } else if (phase == 2) {
            ranking();
            phase = 0;
        } else {
            help();
            phase = 0;
        }
    }
    fini();
    return 0;
}

/**********************************************************************/
