/**********************************************************************/
/* MieruPC Project: LCD Library itcfunc v1.0     Arch Lab. Tokyo Tech */
/* itcfunc.c Version 2009-07-14                                       */
/**********************************************************************/
#include "mierulib.h"

#define BUF_SIZE 128

int width, height, cursorx, cursory, drawpage;
char buf[BUF_SIZE];


/**********************************************************************/
void mpc_waitkey()
{
    while(1);
}

/**********************************************************************/
void sendcmd(char *arg)
{
    volatile char *ptr = (volatile char *) MP_MMAP_LCD;
    for (; *arg; arg++) {
        while (!*ptr);
        *ptr = *arg;
    }
}

/* open and initalize tty, and send SV command if rotate is non-zero */
/**********************************************************************/
int lcd_ttyopen(int rotate)
{
    int i, end;
    volatile int *p;
    if (rotate) {
        sendcmd("SV\r");
        p = (volatile int *) MP_MMAP_TIMER;
        end = (int) *p + 2;
        while (*p < end);
    }

    if (rotate) {
        width = 40; height = 15;
    } else {
        width = 30; height = 20;
    }

    cursorx = cursory = drawpage = 0;
    for (i = 0; i < 8; i++) {
        snprintf(buf, BUF_SIZE, "PS%d\rER\r", i);
        sendcmd(buf);   
    }
    snprintf(buf, BUF_SIZE, "PS0\rDS0\rCLW\rBA%d,%d\r", 
             width * 8, height * 16);
    sendcmd(buf);  
    return 1;
}

/**********************************************************************/
void lcd_ttyclose()
{
    
}

/**********************************************************************/
void lcd_setcolor(int color)
{
    char pattern[16] = "0123456789ABCDEF";
    snprintf(buf, BUF_SIZE, "CS%c%c\r",
             (int) pattern[(color & 0xf0) >> 4],
             (int) pattern[ color & 0x0f]);
    sendcmd(buf);
}

/**********************************************************************/
void lcd_setshowpage(int page)
{
    snprintf(buf, BUF_SIZE, "DS%d\r", page % 8);
    sendcmd(buf);    
}

/**********************************************************************/
void lcd_setdrawpage(int page)
{
    drawpage = page % 8;
    snprintf(buf, BUF_SIZE, "PS%d\r", page % 8);
    sendcmd(buf);
}

/**********************************************************************/
void lcd_copypage(int src, int dest)
{
    snprintf(buf, BUF_SIZE, "BS0,0,%d,0\rBD0,0,%d,1\r", src, dest);
    sendcmd(buf);   
}

/**********************************************************************/
void lcd_cls()
{
    cursorx = cursory = 0;
    sendcmd("ER\r");         
}

/**********************************************************************/
void lcd_setreverse(int flag)
{
    snprintf(buf, BUF_SIZE, "R%c\r", (flag) ? 'V' : 'C');
    sendcmd(buf);
    
}

/**********************************************************************/
void lcd_settransparent(int flag)
{
    snprintf(buf, BUF_SIZE, "M%c\r", (flag) ? 'S' : 'C');
    sendcmd(buf);    
}

/**********************************************************************/
void lcd_locate(int x, int y)
{
    cursorx = x; cursory = y;
    snprintf(buf, BUF_SIZE, "HP%d,%d\r", x, y);
    sendcmd(buf);  
}

/**********************************************************************/
int lcd_printf(char *fmt, ...)
{
    int ret;
    va_list arg;
    va_start(arg, fmt);

    ret = vsnprintf(buf, BUF_SIZE, fmt, arg);
    va_end(arg);

    sendcmd("HW'");
    sendcmd(buf);
    sendcmd("'\r");
    return ret;
}

/**********************************************************************/
void lcd_nextline()
{
    cursorx = 0;
    cursory = (cursory + 1) % height;
    sendcmd("HF\rHR\r");
}

/**********************************************************************/
